#! /usr/bin/env sh

[ "${CD_DNS}" ] || CD_DNS="\
9.9.9.9 \
"
CD_PYTHON_MODULES="\
cd \
rwx \
"

# steps
cd_main () {
    cd_set_environment
    cd_set_dns_resolving ${CD_DNS}
    cd_set_packages_repositories
    cd_set_packages_configuration
    cd_set_https_verification_off
    cd_update_packages_catalog
    cd_install_packages_tools
    cd_install_ca
    cd_copy_ca
    cd_update_ca
    cd_set_https_verification_on
    cd_update_packages_catalog
    cd_upgrade_packages
    cd_install_git
    cd_install_python
    cd_clean_packages_cache
    cd_install_python_modules ${CD_PYTHON_MODULES}
    cd_execute_python_module ${CD_PYTHON_MODULES}
}

# functions

cd_set_environment () {
    cd_step "Set environment"
    #
    CD_CA_PACKAGE="ca-certificates"
    CD_DNS_FILE="/etc/resolv.conf"
    CD_GIT_PACKAGE="git"
    CD_PYTHON_COMMAND="python3"
    #
    case "${CD_OS_NAME}" in
        "debian")
            CD_PYTHON_PACKAGE="python3"
            CD_PYTHON_PACKAGES="/usr/lib/python3/dist-packages"
        ;;
        "alma")
            case "${CD_OS_VERSION}" in
                "8")
                    CD_PYTHON_PACKAGE="python3.11"
                    CD_PYTHON_PACKAGES="/usr/lib64/python3.11/site-packages"
                ;;
                *) cd_error_os "CD_OS_VERSION" ;;
            esac
        ;;
        *) cd_error_os "CD_OS_NAME" ;;
    esac
    # ci / github
    if [ "${GITHUB_ACTIONS}" ] ; then
        CD_SERVER_URL="${GITHUB_SERVER_URL}"
        CD_PROJECTS_GROUP="$(dirname "${GITHUB_REPOSITORY}")"
        CD_PROJECT_NAME="$(basename "${GITHUB_REPOSITORY}")"
        CD_PROJECT_BRANCH="${GITHUB_REF_NAME}"
    # ci / gitlab
    elif [ "${GITLAB_CI}" ] ; then
        CD_SERVER_URL="${CI_SERVER_URL}"
        CD_PROJECTS_GROUP="$(dirname "${CI_PROJECT_PATH}")"
        CD_PROJECT_NAME="$(basename "${CI_PROJECT_PATH}")"
        CD_PROJECT_BRANCH="${CI_COMMIT_BRANCH}"
    # ci / none
    else
        cd_error_ci "ø"
    fi
    #
    [ "${CD_SERVER_URL}" ] || cd_error_ci "CD_SERVER_URL"
    [ "${CD_PROJECTS_GROUP}" ] || cd_error_ci "CD_PROJECTS_GROUP"
    [ "${CD_PROJECT_NAME}" ] || cd_error_ci "CD_PROJECT_NAME"
    [ "${CD_PROJECT_BRANCH}" ] || cd_error_ci "CD_PROJECT_BRANCH"
    #
    CD_PROJECTS_URL="${CD_SERVER_URL}/${CD_PROJECTS_GROUP}"
    CD_PROJECT_URL="${CD_PROJECTS_URL}/${CD_PROJECT_NAME}"
    #
    echo -n "\
CD_OS_NAME=${CD_OS_NAME}
CD_OS_VERSION=${CD_OS_VERSION}
#
CD_DNS_FILE=${CD_DNS_FILE}
#
CD_PYTHON_COMMAND=${CD_PYTHON_COMMAND}
CD_PYTHON_PACKAGES=${CD_PYTHON_PACKAGES}
#
CD_SERVER_URL=${CD_SERVER_URL}
CD_PROJECTS_GROUP=${CD_PROJECTS_GROUP}
CD_PROJECT_NAME=${CD_PROJECT_NAME}
CD_PROJECT_BRANCH=${CD_PROJECT_BRANCH}
#
CD_PROJECTS_URL=${CD_PROJECTS_URL}
CD_PROJECT_URL=${CD_PROJECT_URL}
"
}

cd_set_dns_resolving () {
local server
local text=""
    cd_step "Set DNS resolving"
    for server in "${@}" ; do
        text="${text}nameserver ${server}
"
    done
    cd_write "${CD_DNS_FILE}" "${text}"
}

cd_set_packages_repositories () {
local expression
local file
    cd_step "Set packages repositories"
    case "${CD_OS_NAME}" in
        "debian")
            cd_write "/etc/apt/sources.list" "\
deb https://deb.debian.org/debian bookworm main
deb https://deb.debian.org/debian bookworm-backports main
deb https://deb.debian.org/debian bookworm-updates main
deb https://deb.debian.org/debian-security bookworm-security main
"
        ;;
        "alma")
            file="/etc/yum.repos.d/almalinux.repo"
            for expression in \
                "|^m|# m|" \
                "|^# b|b|" \
            ; do
                sed --in-place "s${expression}g" "${file}" || exit
            done
            cd_cat "${file}"
        ;;
        *) cd_error_os "cd_set_packages_repositories" ;;
    esac
}

cd_set_packages_configuration () {
    cd_step "Set packages configuration"
    case "${CD_OS_NAME}" in
        "debian")
            export DEBIAN_FRONTEND="noninteractive"
            cd_write "/etc/apt/apt.conf.d/apt.conf" "\
Acquire::Check-Valid-Until True;
APT::Get::Show-Versions True;
APT::Install-Recommends False;
APT::Install-Suggests False;
Dir::Etc::SourceParts \"\";
"
        ;;
        "alma") cd_mkdir "/etc/dnf/dnf.conf.d" ;;
        *) cd_error_os "cd_set_packages_configuration" ;;
    esac
}

cd_set_https_verification_off () {
    cd_step "Set HTTPS verification off"
    case "${CD_OS_NAME}" in
        "debian")
            cd_write "/etc/apt/apt.conf.d/https" "\
Acquire::https::Verify-Peer False;
"
        ;;
        "alma")
            cd_write "/etc/dnf/dnf.conf.d/https.conf" "\
sslverify=False
"
        ;;
        *) cd_error_os "cd_set_https_verification_off" ;;
    esac
}

cd_update_packages_catalog () {
    cd_step "Update packages catalog"
    case "${CD_OS_NAME}" in
        "debian") apt-get update || exit ;;
        "alma") dnf makecache || exit ;;
        *) cd_error_os "cd_update_packages_catalog" ;;
    esac
}

cd_install_packages_tools () {
    cd_step "Install packages tools"
    case "${CD_OS_NAME}" in
        "debian") cd_install_package "apt-utils" ;;
        "alma") echo "TODO" ;;
        *) cd_error_os "cd_install_packages_tools" ;;
    esac
}

cd_install_ca () {
    cd_step "Install CA"
    cd_install_package "${CD_CA_PACKAGE}"
}

cd_copy_ca () {
local target
    cd_step "Copy CA"
    case "${CD_OS_NAME}" in
        "debian") target="/usr/local/share/ca-certificates" ;;
        "alma") target="/etc/pki/ca-trust/source/anchors" ;;
        *) cd_error_os "cd_copy_ca" ;;
    esac
    echo "TODO"
}

cd_update_ca () {
    cd_step "Update CA"
    case "${CD_OS_NAME}" in
        "debian") update-ca-certificates || exit ;;
        "alma") update-ca-trust || exit ;;
        *) cd_error_os "cd_update_ca" ;;
    esac
}

cd_set_https_verification_on () {
    cd_step "Set HTTPS verification on"
    case "${CD_OS_NAME}" in
        "debian") cd_rm "/etc/apt/apt.conf.d/https" ;;
        "alma") cd_rm "/etc/dnf/dnf.conf.d/https.conf" ;;
        *) cd_error_os "cd_set_https_verification_on" ;;
    esac
}

cd_upgrade_packages () {
    cd_step "Upgrade packages"
    case "${CD_OS_NAME}" in
        "debian") apt-get upgrade --assume-yes || exit ;;
        "alma") dnf upgrade --assumeyes || exit ;;
        *) cd_error_os "cd_upgrade_packages" ;;
    esac
}

cd_install_git () {
    cd_step "Install Git"
    cd_install_package "${CD_GIT_PACKAGE}"
}

cd_install_python () {
    cd_step "Install Python"
    cd_install_package "${CD_PYTHON_PACKAGE}"
}

cd_clean_packages_cache () {
    cd_step "Clean packages cache"
    case "${CD_OS_NAME}" in
        "debian") apt-get clean || exit ;;
        "alma") dnf clean all || exit ;;
        *) cd_error_os "cd_clean_packages_cache" ;;
    esac
}

cd_install_python_modules () {
local path
local repository
local root
local url
    cd_step "Install Python modules"
    root="$(mktemp --directory)" || exit
    echo "${root}"
    for repository in "${@}" ; do
        url="${CD_PROJECTS_URL}/${repository}"
        echo -n "
${url}
↓
"
        git clone \
        "${url}" "${root}/${repository}" \
        || exit
        path="${root}/${repository}/${repository}"
        echo -n "\
${path}
↓
${CD_PYTHON_PACKAGES}
"
        cp --recursive \
        "${path}" "${CD_PYTHON_PACKAGES}" \
        || exit
    done
    cd_rm "${root}"
}

cd_execute_python_module () {
local self
    cd_step "Execute Python module"
    self="$(realpath "${0}")"
    echo -n "\
${self}
↓
${1}
"
    "${CD_PYTHON_COMMAND}" -m "${1}" "${CD_STEP}" "${self}"
}

# tools

cd_cat () {
    if [ -f "${1}" ] ; then
        echo " ↙ $(realpath "${1}")"
        cat "${1}" || exit
    fi
}

cd_error_ci () {
    echo "× CI: ${1}"
    exit 1
}
cd_error_os () {
    echo "× OS: ${1}"
    exit 2
}

cd_install_package () {
    if [ "${1}" ] ; then
        case "${CD_OS_NAME}" in
            "debian") apt-get install --assume-yes "${1}" || exit ;;
            "alma") dnf install --assumeyes "${1}" || exit ;;
            *) exit 1 ;;
        esac
    fi
}

cd_mkdir () {
    if [ "${1}" ] ; then
        echo "→ ${1}"
        mkdir --parents "${1}" || exit
    fi
}

cd_rm () {
    if [ -e "${1}" ] ; then
        echo "← $(realpath "${1}")"
        rm --recursive "${1}" || exit
    fi
}

cd_step () {
    if [ "${1}" ] ; then
        CD_STEP=$((CD_STEP+1))
        echo -n "\
╭─╌┄┈
│ ${CD_STEP} ${1}
╰─╌┄┈
"
    fi
}

cd_write () {
local file="${1}"
local text="${2}"
    if [ "${file}" ] ; then
        echo -n "${text}" \
        > "${file}" \
        || exit
        cd_cat "${file}"
    fi
}

# main
cd_main
